<?php

namespace Drupal\Tests\acquia_contenthub\Functional;

use Drupal\acquia_contenthub_publisher\PublisherTracker;
use Drupal\file\Entity\File;
use Drupal\node\Entity\NodeType;
use Drupal\Tests\file\Functional\FileFieldTestBase;

/**
 * Uploads a private file to a node and checks that it has not been enqueued for export.
 *
 * @group acquia_contenthub
 */
class FilePrivateTest extends FileFieldTestBase {

  /**
   * Modules to enable.
   *
   * @var array
   */
  public static $modules = ['node_access_test', 'field_test', 'acquia_contenthub', 'acquia_contenthub_publisher'];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  protected function setUp() {
    parent::setUp();
    node_access_test_add_field(NodeType::load('article'));
    node_access_rebuild();
    \Drupal::state()->set('node_access_test.private', TRUE);
    // This test expects unused managed files to be marked as a temporary file.
    $this->config('file.settings')
      ->set('make_unused_managed_files_temporary', TRUE)
      ->save();

    // Set up "Fake" Acquia Content Hub Test Credentials.
    $origin_uuid = '00000000-0000-0001-0000-000000000000';
    $hostname = 'http://plexus-url.remote';
    $api_key = 'AAAAAAAAAAAAAAAAAAAAA';
    $secret_key = 'AAAAAAAAAAAAAAAAAAAAA';
    $configFactory = $this->container->get('config.factory');
    $config = $configFactory->getEditable('acquia_contenthub.admin_settings');
    $config->set('hostname', $hostname);
    $config->set('api_key', $api_key);
    $config->set('secret_key', $secret_key);
    $config->set('origin', $origin_uuid);
    $config->save();
  }

  /**
   * Tests that a private file uploaded to a node is not enqueued for export.
   */
  public function testPrivateFile() {
    $node_storage = $this->container->get('entity_type.manager')->getStorage('node');
    /** @var \Drupal\Core\File\FileSystemInterface $file_system */
    $type_name = 'article';
    $field_name = strtolower($this->randomMachineName());
    $this->createFileField($field_name, 'node', $type_name, ['uri_scheme' => 'private']);

    $test_file = $this->getTestFile('text');
    $nid = $this->uploadNodeFile($test_file, $field_name, $type_name, TRUE, ['private' => TRUE]);
    \Drupal::entityTypeManager()->getStorage('node')->resetCache([$nid]);
    /* @var \Drupal\node\NodeInterface $node */
    $node = $node_storage->load($nid);
    $node_file = File::load($node->{$field_name}->target_id);
    // Ensure the file can be viewed.
    $this->drupalGet('node/' . $node->id());
    $this->assertRaw($node_file->getFilename(), 'File reference is displayed after attaching it');

    // Check that some items have been enqueued for export in the export queue.
    $q = \Drupal::service("queue");
    $qc = $q->get("acquia_contenthub_publish_export");
    $items = $qc->numberOfItems();
    $this->assertNotEqual($items, 0, "{$items} items enqueued.");

    // Find queued items in the publisher tracking table.
    /** @var PublisherTracker $tracker */
    $tracker = \Drupal::service("acquia_contenthub_publisher.tracker");

    // Assert that the node has been enqueued for export.
    $node_record = $tracker->get($node->uuid());
    $this->assertNotEmpty($node_record, "Node Entity is tracked in the Publisher Tracker.");

    // Assert that the private file has NOT been enqueued for export.
    $private_file_record = $tracker->get($node_file->uuid());
    $this->assertEmpty($private_file_record, "Private file is not enqueued for export in the Publisher Tracker.");
  }
}
